/* FILE: simpledemag.h            -*-Mode: c++-*-
 *
 * Average H demag field across rectangular cells.  Simple
 * implementation that does not take advantage of any memory
 * saving or calculation speed improvements possible using
 * symmetries in interaction tensor or realness of data in
 * spatial domain.
 *
 * In 2006 the Periodic Boundary Condition (PBC) functionality
 * was added by Kristof Lebecki (nick KL(m)).
 * With large help of Mike Donahue, of course.
 * See the kl_pbc_util.h file for more information.
 *
 */

#ifndef _KLM_SIMPLEDEMAG_PBC
#define _KLM_SIMPLEDEMAG_PBC

#include "key.h"
#include "energy.h"
#include "fft.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"

/* End includes */

class Klm_SimpleDemag_PBC:public Oxs_Energy {
private:
  // KL(m) parameters
  // Repetition distance for z-direction. In cell meters.
  REAL4m zPeriod_param;
  // Maximal relative cube-cube distance for "image" calculations 
  //  (according to exact cube-cube formalae). 
  // Above this radius dipole-dipole ("dipole") estimation is used.
  // Different values for diagonal (like nXX) and off-diag elements (like nYZ)
  REAL4m max_celldistance_img_diag; 
  REAL4m max_celldistance_img_OFFdiag; 
  // Known averaged accurancy (relative) of the calculations: "image" & "dipole".
  // You should put a value here basing on previous results analysis.
  // This will be used for error statistics information.
  REAL4m error_img_dip_diag;  
  REAL4m error_img_dip_OFFdiag;  
  // How smaller (eg. value 0.1) or larger (10) can be the "cont" error 
  //  (compared to appr. error_img_dip_*)
  REAL4m err_ratio_cont_to_dip;  
  // Maximal number of repetitions in the summation.
  // Used as a safety stopper if the err_ratio_cont_to_dip value is set too low.
  UINT4m max_no_of_sum_elements;  
  // Maximal number of repetitions in the summation.
  // Used for debugging purposes.
  INT4m max_no_of_sum_elements_debug;  
  // Tensor file storage for future use.
  // The tensor will be kept in two files differing by ID (1 or 2). 
  // 1. If the tensor_file_name is a directory (ends with a slash "/"),
  //    the files will we writen in this path using a pattern to distinguish them
  // 2. Otherwise the files will be created basing on a scheme:
  //    tensor_file_name<ID>tensor_file_suffix
  // Allways the ID will be placed before tensor_file_suffix.
  String tensor_file_name;
  String tensor_file_suffix;
  // Script name for showing the progress. With full path.
  // If epmty/not-present: no progress show.
  String progress_script;
  // Normolly we should perform cyclic convolution only (default=1) ??
  BOOL cyclic_convolution; 
  // End of parameters.
  // KL(m) end
  //
  mutable UINT4m xdim;    // Natural size
  mutable UINT4m ydim;
  mutable UINT4m zdim;
  mutable UINT4m totalsize;
  mutable UINT4m pxdim;   // Padded size
  mutable UINT4m pydim;
  mutable UINT4m pzdim;
  mutable UINT4m ptotalsize;
  mutable UINT4m mesh_id;
  // The A## arrays hold demag coefficients, transformed into
  // frequency domain.  These are held long term.  Mx, My, Mz
  // and Hcomp are used as temporary space to convert M into
  // H.
  mutable Oxs_Complex *A00;
  mutable Oxs_Complex *A01;
  mutable Oxs_Complex *A02;
  mutable Oxs_Complex *A11;
  mutable Oxs_Complex *A12;
  mutable Oxs_Complex *A22;
  mutable Oxs_Complex *Mx;
  mutable Oxs_Complex *My;
  mutable Oxs_Complex *Mz;
  mutable Oxs_Complex *Hcomp;
  mutable Oxs_FFT3D fft; // All transforms are same size, so we need
  /// only one Oxs_FFT3D object.
  
  void FillCoefficientArrays(const Oxs_Mesh* mesh) const;
  /// Conceptually const.

  UINT4m NextPowerOfTwo(UINT4m n) const;  // Helper function
  void ReleaseMemory() const; // Conceptually const.

protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Klm_SimpleDemag_PBC(const char* name,  // Child instance id
		  Oxs_Director* newdtr,  // App director
		  const char* argstr);   // MIF input block parameters
  virtual ~Klm_SimpleDemag_PBC() { ReleaseMemory(); }
  virtual BOOL Init();
};


#endif // _KLM_SIMPLEDEMAG_PBC
